==> snippet_manager-8.x-1.3.tar.gz: snippet_manager/tests/src/FunctionalJavascript/EditorTest.php <==
<?php

namespace Drupal\Tests\snippet_manager\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\DrupalSelenium2Driver;
use Drupal\FunctionalJavascriptTests\JavascriptTestBase;

/**
 * Tests the snippet editor.
 *
 * @group snippet_manager
 */
class EditorTest extends JavascriptTestBase {

  /**
   * {@inheritdoc}
   */
  protected $minkDefaultDriverClass = DrupalSelenium2Driver::class;

  /**
   * {@inheritdoc}
   */
  public static $modules = ['snippet_manager_test'];

  /**
   * Tests if the snippet list can be filtered with JavaScript.
   */
  public function testSnippetFilter() {

    $admin_user = $this->drupalCreateUser([
      'administer snippets',
      'use text format snippet_manager_test_basic_format',
    ]);
    $this->drupalLogin($admin_user);

    $this->drupalGet('admin/structure/snippet/alpha/edit/template');

    $this->editorSetValue('');

    // -- Test main buttons.
    $this->editorClickButton('bold');
    $this->editorClickButton('italic');
    $this->editorClickButton('underline');
    $this->editorClickButton('strike-through');
    $this->editorClickButton('list-numbered');
    $this->editorClickButton('list-bullet');

    $expected_output = "<ol>\n  <li></li>\n</ol>\n<ul>\n  <li></li>\n</ul>\n<s></s><u></u><em></em><strong></strong>";
    $this->assertEquals($expected_output, $this->editorGetValue());

    // -- Make sure it works with respect to selection.
    $this->editorSetValue('Test');
    $this->editorSetSelection([0, 0], [0, 4]);
    $this->editorClickButton('bold');

    $this->assertEquals('<strong>Test</strong>', $this->editorGetValue());

    // -- Test 'undo' button.
    $this->editorClickButton('undo');

    $this->assertEquals('Test', $this->editorGetValue());

    // -- Test 'redo' button.
    $this->editorClickButton('redo');
    $this->assertEquals('<strong>Test</strong>', $this->editorGetValue());

    // -- Test 'clear formatting' button.
    $this->editorSetSelection([0, 0], [0, 21]);
    $this->editorClickButton('clear-formatting');
    $this->assertEquals('Test', $this->editorGetValue());

    // -- Test 'enlarge' button.
    $this->editorClickButton('enlarge');
    $this->assertTrue($this->editorGetOption('fullScreen'));
    $this->assertNotVisible('//*[@id = "sm-button-enlarge"]');
    $this->assertVisible('//*[@id = "sm-button-shrink"]');

    // -- Test 'shrink' button.
    $this->editorClickButton('shrink');
    $this->assertFalse($this->editorGetOption('fullScreen'));
    $this->assertVisible('//*[@id = "sm-button-enlarge"]');
    $this->assertNotVisible('//*[@id = "sm-button-shrink"]');

    // -- Test 'mode' dropdown.
    $this->assertEquals('html_twig', $this->editorGetOption('mode'));
    $this->changeMode('text/html');
    $this->assertEquals('text/html', $this->editorGetOption('mode'));
    $this->changeMode('twig');
    $this->assertEquals('twig', $this->editorGetOption('mode'));
    $this->changeMode('javascript');
    $this->assertEquals('javascript', $this->editorGetOption('mode'));
    $this->changeMode('css');
    $this->assertEquals('css', $this->editorGetOption('mode'));

    // Reload the page and make sure the mode has been preserved.
    $this->drupalGet('admin/structure/snippet/alpha/edit/template');
    $this->assertSession()->elementExists('xpath', '//select[@id = "sm-mode"]/option');
    $selected_mode = $this->getSession()->getDriver()->evaluateScript('return jQuery("#sm-mode").val()');
    $this->assertEquals('css', $selected_mode);
    $this->assertEquals('css', $this->editorGetOption('mode'));

    // -- Test inserting of variables.
    $this->editorSetValue('');
    $this->click('//a[@class = "snippet-variable" and text() = "foo"]');
    $this->assertEquals('{{ foo }}', $this->editorGetValue());

    // -- Test CSS editor.
    $this->drupalGet('admin/structure/snippet/alpha/edit/css');
    $this->assertEquals('css', $this->editorGetOption('mode'));
    $this->assertCommonButtons();

    // -- Test JavaScript editor.
    $this->drupalGet('admin/structure/snippet/alpha/edit/js');
    $this->assertEquals('javascript', $this->editorGetOption('mode'));
    $this->assertCommonButtons();

    // -- Test snippet source page.
    $this->drupalGet('admin/structure/snippet/alpha/source');
    $this->assertEquals('text/html', $this->editorGetOption('mode'));
  }

  /**
   * Gets CodeMirror document value.
   */
  protected function editorGetValue() {
    $script = sprintf('return document.getElementsByClassName("CodeMirror")[0].CodeMirror.getValue();');
    return $this->getSession()
      ->getDriver()
      ->evaluateScript($script);
  }

  /**
   * Sets CodeMirror document value.
   */
  protected function editorSetValue($value) {
    $script = sprintf('document.getElementsByClassName("CodeMirror")[0].CodeMirror.setValue("%s");', $value);
    $this->getSession()
      ->getDriver()
      ->executeScript($script);
  }

  /**
   * Sets CodeMirror document selection.
   */
  protected function editorSetSelection($anchor, $head) {
    $script = 'document.getElementsByClassName("CodeMirror")[0].CodeMirror.setSelection({line: %d, ch: %d}, {line: %d, ch: %d});';
    $script = sprintf($script, $anchor[0], $anchor[1], $head[0], $head[1]);
    $this->getSession()
      ->getDriver()
      ->executeScript($script);
  }

  /**
   * Clicks specified editor toolbar button.
   */
  protected function editorClickButton($button) {
    $this->getSession()
      ->getPage()
      ->find('css', '#sm-button-' . $button)
      ->click();
  }

  /**
   * Gets CodeMirror option.
   */
  protected function editorGetOption($option) {
    $script = sprintf('document.getElementsByClassName("CodeMirror")[0].CodeMirror.getOption("%s");', $option);
    return $this->getSession()
      ->getDriver()
      ->evaluateScript($script);
  }

  /**
   * Asserts that the provided element is visible.
   */
  protected function assertVisible($xpath) {
    $is_visible = $this->getSession()
      ->getDriver()
      ->isVisible($xpath);
    $this->assertTrue($is_visible);
  }

  /**
   * Asserts that the provided element is not visible.
   */
  protected function assertNotVisible($xpath) {
    $is_visible = $this->getSession()
      ->getDriver()
      ->isVisible($xpath);
    $this->assertFalse($is_visible);
  }

  /**
   * Changes editor mode.
   */
  protected function changeMode($mode) {
    $this->getSession()
      ->getPage()
      ->find('css', '#sm-mode')
      ->selectOption($mode);
  }

  /**
   * Clicks button or link located by it's XPath query.
   */
  protected function click($xpath) {
    $this->getSession()->getDriver()->click($xpath);
  }

  /**
   * Asserts that common buttons exist.
   */
  protected function assertCommonButtons() {
    $prefix = '//div[@class = "snippet-manager-toolbar"]';
    $this->assertSession()->elementExists('xpath', $prefix . '/*[@id = "sm-button-undo"]');
    $this->assertSession()->elementExists('xpath', $prefix . '/*[@id = "sm-button-redo"]');
    $this->assertSession()->elementExists('xpath', $prefix . '/*[@id = "sm-button-enlarge"]');
    $this->assertSession()->elementExists('xpath', $prefix . '/*[@id = "sm-button-shrink"]');
  }

}

==> snippet_manager-8.x-1.4.tar.gz: snippet_manager/tests/src/FunctionalJavascript/EditorTest.php <==
<?php

namespace Drupal\Tests\snippet_manager\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\DrupalSelenium2Driver;
use Drupal\FunctionalJavascriptTests\JavascriptTestBase;

/**
 * Tests the snippet editor.
 *
 * @group snippet_manager
 */
class EditorTest extends JavascriptTestBase {

  /**
   * {@inheritdoc}
   */
  protected $minkDefaultDriverClass = DrupalSelenium2Driver::class;

  /**
   * {@inheritdoc}
   */
  public static $modules = ['snippet_manager_test', 'file'];

  /**
   * Test callback.
   */
  public function testSnippetEditor() {

    $admin_user = $this->drupalCreateUser([
      'administer snippets',
      'use text format snippet_manager_test_basic_format',
    ]);
    $this->drupalLogin($admin_user);

    $this->drupalGet('admin/structure/snippet/alpha/edit/template');

    $this->editorSetValue('');

    // -- Test main buttons.
    $this->editorClickButton('bold');
    $this->editorClickButton('italic');
    $this->editorClickButton('underline');
    $this->editorClickButton('strike-through');
    $this->editorClickButton('list-numbered');
    $this->editorClickButton('list-bullet');
    $this->editorClickButton('horizontal-rule');

    $expected_output = "<hr/><ul>\n  <li></li>\n</ul>\n<ol>\n  <li></li>\n</ol>\n<s></s><u></u><em></em><strong></strong>";
    $this->assertEquals($expected_output, $this->editorGetValue());

    // -- Make sure it works with respect to selection.
    $this->editorSetValue('Test');
    $this->editorSetSelection([0, 0], [0, 4]);
    $this->editorClickButton('bold');

    $this->assertEquals('<strong>Test</strong>', $this->editorGetValue());

    // -- Test 'undo' button.
    $this->editorClickButton('undo');

    $this->assertEquals('Test', $this->editorGetValue());

    // -- Test 'redo' button.
    $this->editorClickButton('redo');
    $this->assertEquals('<strong>Test</strong>', $this->editorGetValue());

    // -- Test 'clear formatting' button.
    $this->editorSetSelection([0, 0], [0, 21]);
    $this->editorClickButton('clear-formatting');
    $this->assertEquals('Test', $this->editorGetValue());

    // -- Test 'enlarge' button.
    $this->editorClickButton('enlarge');
    $this->assertTrue($this->editorGetOption('fullScreen'));
    $this->assertNotVisible('//*[@data-sm-button = "enlarge"]');
    $this->assertVisible('//*[@data-sm-button = "shrink"]');

    // -- Test 'shrink' button.
    $this->editorClickButton('shrink');
    $this->assertFalse($this->editorGetOption('fullScreen'));
    $this->assertVisible('//*[@data-sm-button = "enlarge"]');
    $this->assertNotVisible('//*[@data-sm-button = "shrink"]');

    // -- Test 'mode' dropdown.
    $this->assertEquals('html_twig', $this->editorGetOption('mode'));
    $this->changeMode('text/html');
    $this->assertEquals('text/html', $this->editorGetOption('mode'));
    $this->changeMode('twig');
    $this->assertEquals('twig', $this->editorGetOption('mode'));
    $this->changeMode('javascript');
    $this->assertEquals('javascript', $this->editorGetOption('mode'));
    $this->changeMode('css');
    $this->assertEquals('css', $this->editorGetOption('mode'));

    // Reload the page and make sure the mode has been preserved.
    $this->drupalGet('admin/structure/snippet/alpha/edit/template');
    $this->assertSession()->elementExists('xpath', '//select[@class = "sm-mode"]/option');
    $selected_mode = $this->getSession()->getDriver()->evaluateScript('return jQuery(".sm-mode").val()');
    $this->assertEquals('css', $selected_mode);
    $this->assertEquals('css', $this->editorGetOption('mode'));

    // -- Test inserting of variables.
    $this->editorSetValue('');
    $this->click('//a[@data-drupal-selector="snippet-variable" and text() = "foo"]');
    $this->assertEquals('{{ foo }}', $this->editorGetValue());

    // -- Test CSS editor.
    $this->drupalGet('admin/structure/snippet/alpha/edit/css');
    $this->assertEquals('css', $this->editorGetOption('mode'));
    $this->assertCommonButtons();

    // -- Test JavaScript editor.
    $this->drupalGet('admin/structure/snippet/alpha/edit/js');
    $this->assertEquals('javascript', $this->editorGetOption('mode'));
    $this->assertCommonButtons();

    // -- Test snippet source page.
    $this->drupalGet('admin/structure/snippet/alpha/source');
    $this->assertEquals('text/html', $this->editorGetOption('mode'));
  }

  /**
   * Gets CodeMirror document value.
   */
  protected function editorGetValue() {
    $script = sprintf('return document.getElementsByClassName("CodeMirror")[0].CodeMirror.getValue();');
    return $this->getSession()
      ->getDriver()
      ->evaluateScript($script);
  }

  /**
   * Sets CodeMirror document value.
   */
  protected function editorSetValue($value) {
    $script = sprintf('document.getElementsByClassName("CodeMirror")[0].CodeMirror.setValue("%s");', $value);
    $this->getSession()
      ->getDriver()
      ->executeScript($script);
  }

  /**
   * Sets CodeMirror document selection.
   */
  protected function editorSetSelection($anchor, $head) {
    $script = 'document.getElementsByClassName("CodeMirror")[0].CodeMirror.setSelection({line: %d, ch: %d}, {line: %d, ch: %d});';
    $script = sprintf($script, $anchor[0], $anchor[1], $head[0], $head[1]);
    $this->getSession()
      ->getDriver()
      ->executeScript($script);
  }

  /**
   * Clicks specified editor toolbar button.
   */
  protected function editorClickButton($button) {
    $this->getSession()
      ->getPage()
      ->find('css', '[data-sm-button="' . $button . '"]')
      ->click();
  }

  /**
   * Gets CodeMirror option.
   */
  protected function editorGetOption($option) {
    $script = sprintf('document.getElementsByClassName("CodeMirror")[0].CodeMirror.getOption("%s");', $option);
    return $this->getSession()
      ->getDriver()
      ->evaluateScript($script);
  }

  /**
   * Asserts that the provided element is visible.
   */
  protected function assertVisible($xpath) {
    $is_visible = $this->getSession()
      ->getDriver()
      ->isVisible($xpath);
    $this->assertTrue($is_visible);
  }

  /**
   * Asserts that the provided element is not visible.
   */
  protected function assertNotVisible($xpath) {
    $is_visible = $this->getSession()
      ->getDriver()
      ->isVisible($xpath);
    $this->assertFalse($is_visible);
  }

  /**
   * Changes editor mode.
   */
  protected function changeMode($mode) {
    $this->getSession()
      ->getPage()
      ->find('css', '.sm-mode')
      ->selectOption($mode);
  }

  /**
   * Clicks button or link located by it's XPath query.
   */
  protected function click($xpath) {
    $this->getSession()->getDriver()->click($xpath);
  }

  /**
   * Asserts that common buttons exist.
   */
  protected function assertCommonButtons() {
    $prefix = '//div[@class = "snippet-manager-toolbar"]';
    $this->assertSession()->elementExists('xpath', $prefix . '/*[@data-sm-button="undo"]');
    $this->assertSession()->elementExists('xpath', $prefix . '/*[@data-sm-button="redo"]');
    $this->assertSession()->elementExists('xpath', $prefix . '/*[@data-sm-button="enlarge"]');
    $this->assertSession()->elementExists('xpath', $prefix . '/*[@data-sm-button="shrink"]');
  }

}
